@@ -38,6 +38,7 @@ from facebookads.mixins import (
 import hashlib
 import collections
 import json
+import six
 
 
 class EdgeIterator(object):
@@ -73,15 +74,8 @@ class EdgeIterator(object):
                 is the parameter name and its value is a string or an object
                 which can be JSON-encoded.
         """
-        if fields is None:
-            fields = []
-
-        fields = ','.join([
-            field for field in (
-                fields if fields
-                else target_objects_class.get_default_read_fields()
-            )
-        ])
+        fields = ','.join(fields or
+                          target_objects_class.get_default_read_fields())
 
         self._params = {} if params is None else params.copy()
 
@@ -256,14 +250,11 @@ class AbstractObject(collections.MutableMapping):
         if isinstance(data, AbstractObject):
             data = data.export_data()
         elif isinstance(data, dict):
-            for key, value in data.items():
-                if value is None:
-                    del data[key]
-                else:
-                    data[key] = self.export_value(value)
+            data = dict((k, self.export_value(v))
+                        for k, v in data.items()
+                        if v is not None)
         elif isinstance(data, list):
-            for i, value in enumerate(data):
-                data[i] = self.export_value(value)
+            data = [self.export_value(v) for v in data]
         return data
 
     def export_data(self):
@@ -296,13 +287,9 @@ class AbstractCrudObject(AbstractObject):
         super(AbstractCrudObject, self).__init__()
 
         self._changes = {}
-
-        if fbid is not None:
-            self[self.__class__.Field.id] = str(fbid)
-
-        if parent_id is not None:
-            self.parent_id = str(parent_id)
-        self.api = api
+        self[self.Field.id] = fbid
+        self._parent_id = parent_id
+        self._api = api
 
     def __setitem__(self, key, value):
         """Sets an item in this CRUD object while maintaining a changelog."""
@@ -316,7 +303,8 @@ class AbstractCrudObject(AbstractObject):
 
     def __delitem__(self, key):
         del self._data[key]
-        del self._changes[key]
+        if key in self._changes:
+            del self._changes[key]
 
     def __eq__(self, other):
         """Two objects are the same if they have the same fbid."""
@@ -334,26 +322,47 @@ class AbstractCrudObject(AbstractObject):
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    @classmethod
+    def get_by_ids(cls, ids, params=None, fields=None):
+        params = {} if params is None else params.copy()
+        if not fields:
+            params['fields'] = cls.get_default_read_fields()
+        else:
+            params['fields'] = ','.join(fields)
+
+        params['ids'] = ','.join(map(str, ids))
+        response = FacebookAdsApi.get_default_api().call(
+            FacebookAdsApi.HTTP_METHOD_GET,
+            ['/'],
+            params=params,
+        )
+        result = []
+        for fbid, data in response.json().items():
+            obj = cls(fbid, api=FacebookAdsApi.get_default_api())
+            obj._set_data(data)
+            result.append(obj)
+        return result
+
     # Getters
 
     def get_id(self):
         """Returns the object's fbid if set. Else, it returns None."""
-        if self.__class__.Field.id in self:
-            return self[self.__class__.Field.id]
+        if self.Field.id in self:
+            return self[self.Field.id]
         else:
             return None
 
     def get_parent_id(self):
         """Returns the object's parent's id."""
-        return self.parent_id
+        return self._parent_id or FacebookAdsApi.get_default_account_id()
 
     def get_api(self):
         """
         Returns the api associated with the object. If None, returns the default
         api.
         """
-        if self.api is not None:
-            return self.api
+        if self._api is not None:
+            return self._api
         else:
             return FacebookAdsApi.get_default_api()
 
@@ -364,15 +373,15 @@ class AbstractCrudObject(AbstractObject):
             FacebookBadObjectError if the object does not have an id.
         """
         if (
-            self.__class__.Field.id not in self or
-            self[self.__class__.Field.id] is None
+            self.Field.id not in self or
+            self[self.Field.id] is None
         ):
             raise FacebookBadObjectError(
                 "%s object needs an id for this operation."
                 % self.__class__.__name__
             )
 
-        return self[self.__class__.Field.id]
+        return self.get_id()
 
     def get_parent_id_assured(self):
         """Returns the object's parent's fbid.
@@ -380,13 +389,13 @@ class AbstractCrudObject(AbstractObject):
         Raises:
             FacebookBadObjectError if the object does not have a parent id.
         """
-        if self.parent_id is None:
+        if self.get_parent_id() is None:
             raise FacebookBadObjectError(
                 "%s object needs a parent_id for this operation."
                 % self.__class__.__name__
             )
 
-        return self.parent_id
+        return self.get_parent_id()
 
     def get_api_assured(self):
         """Returns the fbid of the object.
@@ -445,7 +454,7 @@ class AbstractCrudObject(AbstractObject):
 
     def clear_id(self):
         """Clears the object's fbid."""
-        del self[self.__class__.Field.id]
+        del self[self.Field.id]
         return self
 
     def get_node_path(self):
@@ -485,7 +494,7 @@ class AbstractCrudObject(AbstractObject):
             self if not a batch call.
             the return value of batch.add if a batch call.
         """
-        if self.__class__.Field.id in self:
+        if self.get_id() is not None:
             raise FacebookBadObjectError(
                 "This %s object was already created." % self.__class__.__name__
             )
@@ -522,6 +531,7 @@ class AbstractCrudObject(AbstractObject):
                 files=files,
             )
             self._set_data(response.json())
+            self._clear_history()
 
             return self
 
@@ -553,18 +563,11 @@ class AbstractCrudObject(AbstractObject):
             self if not a batch call.
             the return value of batch.add if a batch call.
         """
+        params = {} if params is None else params.copy()
         if not fields:
-            fields = self.get_default_read_fields()
-
-        if fields:
-            params = {} if params is None else params.copy()
-
-            fields = ','.join(
-                fields if fields
-                else self.get_default_read_fields()
-            )
-
-            params['fields'] = fields
+            params['fields'] = self.get_default_read_fields() or None
+        else:
+            params['fields'] = ','.join(fields)
 
         if batch is not None:
             def callback_success(response):
@@ -717,7 +720,7 @@ class AbstractCrudObject(AbstractObject):
         Calls remote_create method if object has not been created. Else, calls
         the remote_update method.
         """
-        if self.__class__.Field.id in self:
+        if self.Field.id in self:
             return self.remote_update(*args, **kwargs)
         else:
             return self.remote_create(*args, **kwargs)
@@ -1069,7 +1072,7 @@ class AdAccountGroupAccount(AbstractCrudObject):
 
     def get_ad_account(self):
         """Returns an AdAccount object with the same account id."""
-        return AdAccount(fbid='act_' + self[self.__class__.Field.account_id])
+        return AdAccount(fbid='act_' + self[self.Field.account_id])
 
 
 class AdAccountGroupUser(AbstractCrudObject):
@@ -1097,7 +1100,7 @@ class AdAccountGroupUser(AbstractCrudObject):
 
     def get_ad_user(self):
         """Returns an AdUser object with the same account id."""
-        return AdUser(fbid=self[self.__class__.Field.uid])
+        return AdUser(fbid=self[self.Field.uid])
 
 
 class AdCampaign(HasStatus, HasObjective, CanArchive, AbstractCrudObject):
@@ -1156,10 +1159,10 @@ class AdSet(HasStatus, CanArchive, AbstractCrudObject):
         updated_time = 'updated_time'
 
     class BidType(object):
-        absolute_ocpm = 'absolute_ocpm'
+        absolute_ocpm = 'ABSOLUTE_OCPM'
         cpc = 'CPC'
         cpm = 'CPM'
-        multi_premium = 'multi_premium'
+        multi_premium = 'MULTI_PREMIUM'
 
     class PacingType(object):
         day_parting = 'day_parting'
@@ -1287,6 +1290,7 @@ class AdCreative(AbstractCrudObject):
         object_type = 'object_type'
         object_url = 'object_url'
         preview_url = 'preview_url'
+        thumbnail_url = 'thumbnail_url'
         title = 'title'
         url_tags = 'url_tags'
         video_id = 'video_id'
@@ -1329,9 +1333,9 @@ class AdImage(CannotUpdate, AbstractCrudObject):
             if key in self._changes:
                 del self._changes[key]
 
-        self._data[self.__class__.Field.id] = '%s:%s' % (
+        self._data[self.Field.id] = '%s:%s' % (
             self.get_parent_id_assured()[4:],
-            self[self.__class__.Field.hash],
+            self[self.Field.hash],
         )
 
         return self
@@ -1350,11 +1354,11 @@ class AdImage(CannotUpdate, AbstractCrudObject):
         not have the files argument but requires the 'filename' property to be
         defined.
         """
-        if self[self.__class__.Field.filename] is None:
+        if self[self.Field.filename] is None:
             raise FacebookBadObjectError(
                 "AdImage required a filename to be defined."
             )
-        filename = self[self.__class__.Field.filename]
+        filename = self[self.Field.filename]
         open_file = open(filename, 'rb')
         return_val = super(AdImage, self).remote_create(
             files={filename: open_file},
@@ -1368,7 +1372,7 @@ class AdImage(CannotUpdate, AbstractCrudObject):
 
     def get_hash(self):
         """Returns the image hash to which AdCreative's can refer."""
-        return self[self.__class__.Field.hash]
+        return self[self.Field.hash]
 
 
 class AdPreview(AbstractObject):
@@ -1392,7 +1396,7 @@ class AdPreview(AbstractObject):
 
     def get_html(self):
         """Returns the preview html."""
-        return self[self.__class__.Field.body]
+        return self[self.Field.body]
 
 
 class AdCreativePreview(AdPreview):
@@ -1523,6 +1527,8 @@ class CustomAudience(AbstractCrudObject):
             for user in users:
                 if schema == cls.Schema.email_hash:
                     user = user.strip(" \t\r\n\0\x0B.").lower()
+                if isinstance(user, six.text_type):
+                    user = user.encode('utf8')  # required for hashlib
                 hashed_users.append(hashlib.sha256(user).hexdigest())
 
         payload = {
@@ -1763,10 +1769,10 @@ class ReachFrequencyPrediction(AbstractCrudObject):
             self.Field.budget: budget,
             self.Field.reach: reach,
             self.Field.impression: impression,
-            self.Field.action: self.__class__.Action.reserve,
+            self.Field.action: self.Action.reserve,
         }
         # Filter out None values.
-        params = {k: v for (k, v) in params.items() if v is not None}
+        params = dict((k, v) for k, v in params.items() if v is not None)
 
         response = self.get_api_assured().call(
             FacebookAdsApi.HTTP_METHOD_POST,
@@ -1779,7 +1785,7 @@ class ReachFrequencyPrediction(AbstractCrudObject):
     def cancel(self):
         params = {
             self.Field.prediction_id: self.get_id_assured(),
-            self.Field.action: self.__class__.Action.cancel,
+            self.Field.action: self.Action.cancel,
         }
         self.get_api_assured().call(
             FacebookAdsApi.HTTP_METHOD_POST,
@@ -1892,3 +1898,28 @@ class Transaction(AbstractObject):
     @classmethod
     def get_endpoint(cls):
         return 'transactions'
+
+
+class AutoComplete(AbstractCrudObject):
+
+    class Type(object):
+        adcountry = 'adcountry'
+        adregion = 'adregion'
+        adcity = 'adcity'
+        adeducationschool = 'adeducationschool'
+        adeducationmajor = 'adeducationmajor'
+        adlocale = 'adlocale'
+        adworkemployer = 'adworkemployer'
+        adworkposition = 'adworkposition'
+        adkeyword = 'adkeyword'
+        adzipcode = 'adzipcode'
+        adgeolocation = 'adgeolocation'
+
+    @classmethod
+    def get_endpoint(cls):
+        return 'search'
+
+    def get_node_path(self):
+        return (
+            self.get_endpoint(),
+        )
