@@ -124,6 +124,7 @@ class FacebookAdsTestCase(unittest.TestCase):
             objects.AdGroup.Field.creative: {
                 objects.AdGroup.Field.Creative.creative_id: creative_id,
             },
+            objects.AdGroup.Field.status: objects.AdGroup.Status.paused,
         })
 
         return ad_group
@@ -147,11 +148,11 @@ class AbstractCrudObjectTestCase(AbstractObjectTestCase):
         '''Tests if object can be created.
         It asserts that id is empty before creation, and populated after.
         '''
-        assert subject.__class__.Field.id not in subject
+        assert subject[subject.Field.id] is None
 
         subject.remote_create()
 
-        assert subject.__class__.Field.id in subject
+        assert subject[subject.Field.id] is not None
 
     @classmethod
     def assert_can_read(cls, subject):
@@ -159,7 +160,7 @@ class AbstractCrudObjectTestCase(AbstractObjectTestCase):
         Reads default fields of subject and sees if subject matches with its
         mirror in all the subject's fields.
         '''
-        assert subject.__class__.Field.id in subject
+        assert subject.Field.id in subject
 
         fields_to_read = subject.get_default_read_fields()
 
@@ -175,7 +176,7 @@ class AbstractCrudObjectTestCase(AbstractObjectTestCase):
 
     @classmethod
     def assert_can_update(cls, subject):
-        assert subject.__class__.Field.id in subject
+        assert subject.Field.id in subject
 
         subject.remote_update()
 
@@ -187,20 +188,20 @@ class AbstractCrudObjectTestCase(AbstractObjectTestCase):
 
     @classmethod
     def assert_can_delete(cls, subject):
-        assert subject.__class__.Field.id in subject
+        assert subject.Field.id in subject
 
         subject.remote_delete()
 
-        assert subject.__class__.Field.id not in subject
+        assert subject.Field.id not in subject
 
     @classmethod
     def assert_can_archive(cls, subject):
-        assert subject.__class__.Field.id in subject
+        assert subject.Field.id in subject
 
         subject.remote_archive()
 
-        subject.remote_read(fields=[subject.__class__.Field.status])
-        assert subject[subject.__class__.Field.status] == 'ARCHIVED'
+        subject.remote_read(fields=[subject.Field.status])
+        assert subject[subject.Field.status] == 'ARCHIVED'
 
 
 class AdUserTestCase(AbstractCrudObjectTestCase):
@@ -269,6 +270,57 @@ class AdCampaignTestCase(AbstractCrudObjectTestCase):
         self.assert_can_delete(self.subject)
 
 
+class GetByIDsTestCase(AbstractCrudObjectTestCase):
+    def runTest(self):
+        self.campaign1 = self.new_test_ad_campaign()
+        self.campaign1['name'] = "Campaign 1"
+        self.campaign1.remote_create()
+        self.campaign2 = self.new_test_ad_campaign()
+        self.campaign2['name'] = "Campaign 2"
+        self.campaign2.remote_create()
+
+        campaigns = objects.AdCampaign.get_by_ids(
+            ids=[self.campaign1.get_id(), self.campaign2.get_id()],
+            fields=['name']
+        )
+
+        assert len(campaigns) == 2
+        assert (
+            campaigns[0]['name'] == "Campaign 1" and
+            campaigns[1]['name'] == "Campaign 2"
+        ) or (
+            campaigns[0]['name'] == "Campaign 2" and
+            campaigns[1]['name'] == "Campaign 1"
+        )
+
+        campaigns[0].remote_delete()
+        campaigns[1].remote_delete()
+
+
+class DefaultReadFieldsTestCase(AbstractCrudObjectTestCase):
+    def runTest(self):
+        old_default_read_fields = objects.AdCampaign.get_default_read_fields()
+
+        campaign = self.new_test_ad_campaign()
+        campaign.remote_create()
+        same_campaign = objects.AdCampaign(campaign.get_id())
+        same_campaign.remote_read()
+        assert objects.AdCampaign.Field.status not in same_campaign
+
+        campaigns = objects.AdCampaign.get_by_ids(ids=[campaign.get_id()])
+        assert objects.AdCampaign.Field.status not in campaigns[0]
+
+        objects.AdCampaign.set_default_read_fields([
+            objects.AdCampaign.Field.status,
+        ])
+        same_campaign.remote_read()
+        assert objects.AdCampaign.Field.status in same_campaign
+
+        campaigns = objects.AdCampaign.get_by_ids(ids=[campaign.get_id()])
+        assert objects.AdCampaign.Field.status in campaigns[0]
+
+        objects.AdCampaign.set_default_read_fields(old_default_read_fields)
+
 class AdSetTestCase(AbstractCrudObjectTestCase):
 
     @classmethod
